#
# This example reads a test pulse from the Timepix4 over slow-control
#
import grpc
import queue
import os
import numpy as np
import tempfile
from pathlib import Path

try:
    import matplotlib.pyplot as plt
except ImportError:
    import sys
    print("matplotlib is required for this example", file=sys.stderr)
    sys.exit(1)

from spidr4 import stream, tpx4tools, rpc, bitfield
import example_config as e

with grpc.insecure_channel(e.spidr4host()) as channel:
    # Get the services
    # ------------------------------------------------------------------------------------------------------
    control = rpc.ControlInfoStub(channel)
    tpx4 = rpc.Timepix4Stub(channel)

    # Reset the pixel chips (will also load the default configuration)
    # ------------------------------------------------------------------------------------------------------
    control.ResetPixelChips(rpc.EMPTY)

    # Configure the test pulse
    # ------------------------------------------------------------------------------------------------------
    tpConfig = rpc.Tpx4TestPulseConfig(
        idx=e.spidr4chip(),
        count=1,
        period_on_us=1,
        period_off_us=1,
        digital=True,
        link_shutter=True
    )
    tpx4.TestPulseSetConfig(tpConfig)

    # Configure event-based readout
    # ------------------------------------------------------------------------------------------------------
    readoutCfg = rpc.Tpx4ReadoutConfig(
        idx=e.spidr4chip(),
        mode=rpc.TPX4_READOUT_TOA_TOT,
        pc24b_thr=1
    )
    tpx4.ReadoutSetConfig(readoutCfg)

    # Configure Pixel Matrix
    # ------------------------------------------------------------------------------------------------------
    pixel_cfg = tpx4tools.PixelConfig(dac=0, power_enable=True, tp_enable=False, mask=False).word
    pixel_cfg_tp = tpx4tools.PixelConfig(dac=0, power_enable=True, tp_enable=True, mask=False).word
    spixel_cfg = np.full((512, 448), pixel_cfg, dtype=np.uint8)

    script_dir = os.path.dirname(os.path.abspath(__file__))
    img = np.load(os.path.join(script_dir, "timepix_test_image.npz"))['img']
    for y in range(0, 512):
        for x in range(0, 448):
          if img[y, x]:
            spixel_cfg[y, x] = pixel_cfg_tp

    config_blob = tpx4tools.logic2chip_cfg_matrix(spixel_cfg)

    tpx4.ConfigPixels(
        rpc.Tpx4PixelConfig(
            idx=e.spidr4chip(),
            config=config_blob.tobytes()
        )
    )


    # Configure shutter
    # ------------------------------------------------------------------------------------------------------
    tpx4.ShutterSetConfig(
        rpc.Tpx4ShutterConfig(
            idx=0,
            mode=rpc.TPX4_SHUTTER_MODE_PROG_REPEAT,
            input=rpc.TPX4_SHUTTER_INPUT_SLOW_CONTROL,
            prog_open_us=20,
            prog_close_us=20
        )
    )



    # Enable the test-pulse
    # ------------------------------------------------------------------------------------------------------
    tpx4.TestPulseEnable(rpc.ChipIndex(idx=e.spidr4chip()))
    tpx4.T0Sync(rpc.ChipIndex(idx=e.spidr4chip()))
#    input(">")

    # Start packet reader thread
    q = queue.Queue()
    prt = stream.packet_read_thread(tpx4, q, chipIndex=e.spidr4chip(), forward_empty=False)

    # Test-pulse start
    # ------------------------------------------------------------------------------------------------------
    tpx4.TestPulseStart(rpc.ChipIndex(idx=e.spidr4chip()))

    # Retrieve the packets
    # ------------------------------------------------------------------------------------------------------
    counters=np.zeros(shape=(512, 448), dtype=np.uint16)

    for data in stream.queue_generator(q, 2):
        pkt = tpx4tools.decode_dd_packet(data)
        if isinstance(pkt, tpx4tools.ToAToTPacket):
            # print(f"Hit at: {pkt.Top}, {pkt.EoC}, {pkt.SPGroup}, {pkt.SPixel} {pkt.Pixel}")
            x, y = tpx4tools.chip_coords2logic(pkt.Top, pkt.EoC, pkt.SPGroup, pkt.SPixel, pkt.Pixel)
            counters[y, x] += 1

    # Stop the packet read thread
    prt.stop()

    # Show received hits
    # ------------------------------------------------------------------------------------------------------
    plt.gcf().set_size_inches(9,9)
    plt.imshow(counters, origin='lower')
    plt.tight_layout()

    store_as = Path(tempfile.gettempdir()) / 'tpx4_sc_testpulse.png'

    plt.savefig(store_as)
    print(f"Saved image to {store_as}")



