#
# This example reads a test pulse from the Timepix4 over 10 Gb Ethernet
#
import grpc
import numpy as np
from spidr4 import rpc, tpx4tools, utils
import time
import sys
import example_config as e

if len(sys.argv) != 3:
  print("./tpx4_xgbe_testpulse_local_daq.py <spidr4_ip> <10gbe network interface>")
  sys.exit(1)

spidr4_ip = sys.argv[1]
iface2find = sys.argv[2]

XGBE_PORT=8192

# Find network interface information
# -----------------------------------------------------------------------------------------------------------
# Get network card MAC address and IP address
xgbe_host_mac, xgbe_host_ip = utils.get_nic_info(iface2find)

# make-up an 10gbe IP for the spidr4 module, as long as it is not the same as xgbe_host_ip
xgbe_spidr_ip = utils.inc_ip(xgbe_host_ip)

  

with grpc.insecure_channel(e.spidr4host()) as channel:
    # Get the services
    # ------------------------------------------------------------------------------------------------------
    ctrl = rpc.ControlInfoStub(channel)
    tpx4 = rpc.Timepix4Stub(channel)
    datastream = rpc.DataStreamStub(channel)

    # Reset the pixel chips (will also load the default configuration)
    # ------------------------------------------------------------------------------------------------------
    ctrl.ResetPixelChips(rpc.EMPTY)

    # Configure the output
    # ------------------------------------------------------------------------------------------------------
    datastream.ConfigXGbe(rpc.XGbeConfig(
        idx=e.spidr4chip(),
        primary=rpc.XGbeLinkConfig(
            source_ip=xgbe_spidr_ip,
            dest_ip=xgbe_host_ip,
            dest_mac=xgbe_host_mac,
            port=8192
        )
    ))

    # Configure the test pulse
    # ------------------------------------------------------------------------------------------------------
    tpConfig = rpc.Tpx4TestPulseConfig(
        idx=e.spidr4chip(),
        count=1,
        period_on_us=1,
        period_off_us=1,
        digital=True,
        link_shutter=True
    )
    tpx4.TestPulseSetConfig(tpConfig)

    # Configure event-based readout
    # ------------------------------------------------------------------------------------------------------
    readoutCfg = rpc.Tpx4ReadoutConfig(
        idx=e.spidr4chip(),
        mode=rpc.TPX4_READOUT_TOA_TOT,
        pc24b_thr=1
    )
    tpx4.ReadoutSetConfig(readoutCfg)

    # Configure Pixel Matrix
    # ------------------------------------------------------------------------------------------------------
    pixel_cfg = tpx4tools.PixelConfig(dac=31, power_enable=True, tp_enable=False, mask=False).word
    pixel_cfg_tp = tpx4tools.PixelConfig(dac=31, power_enable=True, tp_enable=True, mask=False).word
    spixel_cfg = np.full((512, 448), pixel_cfg, dtype=np.uint8)

    img = np.load("timepix_test_image.npz")['img']
    for y in range(0, 512):
        for x in range(0, 448):
            if img[y,x]:
                spixel_cfg[y, x] = pixel_cfg_tp

    config_blob = tpx4tools.logic2chip_cfg_matrix(spixel_cfg)

    tpx4.ConfigPixels(
        rpc.Tpx4PixelConfig(
            idx=e.spidr4chip(),
            config=config_blob.tobytes()
        )
    )

    # Configure shutter
    # ------------------------------------------------------------------------------------------------------
    tpx4.ShutterSetConfig(
        rpc.Tpx4ShutterConfig(
            idx=e.spidr4chip(),
            mode=rpc.TPX4_SHUTTER_MODE_PROG_SINGLE,
            input=rpc.TPX4_SHUTTER_INPUT_SLOW_CONTROL,
            prog_open_us=10,
            prog_close_us=10
        )
    )

    # Enable the test-pulse
    # ------------------------------------------------------------------------------------------------------
    tpx4.TestPulseEnable(rpc.ChipIndex(idx=e.spidr4chip()))

    # Test-pulse start
    # ------------------------------------------------------------------------------------------------------
    tpx4.TestPulseStart(rpc.ChipIndex(idx=e.spidr4chip()))

    # Wait for some data...
    time.sleep(1)
    
    # Disable data acquisition.
    # ------------------------------------------------------------------------------------------------------
    datastream.ConfigNone(rpc.ChipIndex(idx=e.spidr4chip()))
